@@ -2,16 +2,19 @@ package Finance::Bank::Norisbank;
 
 =head1 NAME
 
-Finance::Bank::Norisbank - Automated account interaction for customers of Norisbank GMBH.
+Finance::Bank::Norisbank - Automated account interaction for customers of Norisbank AG.
 
 =cut
 
-$VERSION=0.01;
+$VERSION=0.02;
+$DEBUG=0;
+use vars '$DEBUG';
 use warnings;
 use strict;
+use Carp;
 use HTML::TreeBuilder;
 use URI;
-use URI::QueryParam;
+use File::Spec;
 BEGIN {
     if (0 && require Data::Dump::Streamer) {
 	'Data::Dump::Streamer'->import('Dumper');
@@ -22,10 +25,9 @@ BEGIN {
 }
 use Time::Local;
 use WWW::Mechanize;
-use URI::URL;
 use HTML::TableExtract;
-use POSIX; # for ceil and floor
-use Locale::Object; # for visa/foreign currency
+use POSIX 'ceil', 'floor'; # could reimplement easily enough, but should be pretty universal.
+use Storable;
 $|=1;
 
 =item new
@@ -53,8 +55,8 @@ sub new {
     
     my $agent = WWW::Mechanize->new(
 	autocheck=>1, # Die on fetch errors
-	
     ) or die "Couldn't create WWW::Mechanize agent";
+    
     $agent->env_proxy;
     $callback ||= sub {};
     my $self = bless {accountnr=>$accountnr, pin=>$pin, agent=>$agent, callback=>$callback}, $class;
@@ -62,6 +64,10 @@ sub new {
     $self->login;
 }
 
+sub dprint {
+    print @_ if $DEBUG;
+}
+
 sub status {
     my $self=shift;
     $self->{callback}->($self, @_);
@@ -72,10 +78,11 @@ sub login {
     my $agent=$self->{agent};
     
     $self->status('Getting login form...');
-    $agent->get('https://onlinebanking.norisbank.de/login/login.jsp');
+    $agent->get('https://onlinebanking.norisbank.de/norisbank/login.do?method=login');
     $self->status('Logging in...');
 
-    $agent->form_name('login');
+    # Defensiveness -- it used to be login, now it's loginForm -- but in both cases, it's the only form on the page.
+    $agent->form_number(0);
     $agent->current_form->value('kontonummer', $self->{accountnr});
     $agent->current_form->value('pin', $self->{pin});
     
@@ -88,10 +95,6 @@ sub login {
 
     $self->{last_logged_in}=time;
 
-    $self->{page} = $self->get_page_name($agent->content);
-#    $self->{frame} = 1 if($self->{page});
-    # We are now on the BankStatus page.
-    
     $self->status('Logged in.');
 
     return $self;
@@ -132,7 +135,7 @@ sub parse_login_error {
 }
 
 
-=item new
+=item get_transactions
 
   $account->get_transactions($datefrom, $dateto);
 
@@ -161,44 +164,40 @@ Return in scalar context is not defined, and subject to change without notice.
 sub get_transactions
 {
     my $self     = shift;
-    my $datefrom = shift;
-    my $dateto   = shift;
+    my $datefrom = shift||''; # Avoid undef warnings.
+    my $dateto   = shift||'';
     my $agent    = $self->{agent};
     
-    print "($datefrom, $dateto)\n";
+    dprint "($datefrom, $dateto)\n";
 
     $dateto = timelocal(0,0,0,$dateto->[1],$dateto->[0]-1,104) 
       if ref($dateto) eq "ARRAY";
     $datefrom = timelocal(0,0,0,$datefrom->[1],$datefrom->[0]-1,104) 
       if ref($datefrom) eq "ARRAY";
 
-    print "($datefrom, $dateto)\n";
+    dprint "($datefrom, $dateto)\n";
     
     $dateto = time() if !$dateto;
-    print "($datefrom, $dateto)\n";
+    dprint "($datefrom, $dateto)\n";
     $datefrom = $dateto-89*24*60*60 if !$datefrom;
-    print "($datefrom, $dateto)\n";
+    dprint "($datefrom, $dateto)\n";
     
     foreach ($datefrom, $dateto) {
 	local $ENV{TZ}="Europe/Rome";
-	print scalar localtime $_;
+	scalar localtime $_ if $DEBUG;
 	my (undef, undef, undef, $day, $mon) = localtime($_);
 	$_=sprintf("%02d\.%02d", $day, $mon+1);
 	
-	print " => $_\n";
+	dprint " => $_\n";
     }
     
     $self->status('Getting transactions list...', 0, 10);
-    if($self->{page} ne 'bankstatus')
-    {
-	$self->find_page('bankstatus', $agent->content);
-    }
     $self->status('Getting transactions list...', 1, 10);
 
     $agent->follow('1') or
         warn $self->parse_error($agent);
     $self->status('Getting transactions list...', 2, 10);
-    $agent->follow('Auswahl Girokonten') or
+    $agent->follow('Girokonten') or
         warn $self->parse_error($agent);
     $self->status('Getting transactions list...', 3, 10);
     
@@ -218,9 +217,12 @@ sub get_transactions
 
 #    print $agent->content, "\n\n";
     $agent->form_number(0);
-    $agent->current_form->value('datumvon', $datefrom);
-    $agent->current_form->value('datumbis', $dateto);
-    $agent->click('');
+    $agent->current_form->value('von', $datefrom);
+    $agent->current_form->value('bis', $dateto);
+#    $agent->click('');
+    # Norisbank is evil!  We hateses them, we does, yess, yessss.
+    $agent->current_form->action('https://onlinebanking.norisbank.de/norisbank/umsatzauskunft.do?method=giroUmsatzauskunft');
+    $agent->submit;
     $self->status('Getting transactions list...', 8, 10);
     $agent->follow(1) or
         warn $self->parse_error($agent);
@@ -242,25 +244,26 @@ sub get_transactions
     
     my @bghabens = $tree->look_down(_tag=>'a');
     
-    print $tree->dump;
+    $tree->dump if $DEBUG;
     
     if (!@bghabens) {
-	print $agent->content;
+	dprint $agent->content;
 	die "Huh, no transactions?";
     }
     
-    my @trs = map {$_->parent->parent} @bghabens;
+    # We want only the TR that contains the a tag directly, thus the scalar.
+    my @trs = map {scalar $_->look_up('_tag' => 'tr')} @bghabens;
     {
 	my %seen;
 
 	@trs = grep {
-#	    print $_->address, " ", $seen{$_->address}||0, "\n";
+#	    dprint $_->address, " ", $seen{$_->address}||0, "\n";
 	    !$seen{$_->address}++
 	} @trs;
     }
     
-    print "Count \@trs: ".@trs."\n";
-    print Dumper \@trs;
+    dprint "Count \@trs: ".@trs."\n";
+    dprint Dumper \@trs;
     
     $self->status("Parsing transactions...", 0, 0+@trs);
 
@@ -270,73 +273,28 @@ sub get_transactions
 	my $tr=$_;
 	my %trans;
 	
-#	print "\n", $tr->address, "\n";
-#	print $tr->dump;
+	dprint "\n", $tr->address, "\n";
+	$tr->dump if $DEBUG;
 	
+	$trans{address}         = $tr->address;
 	$trans{bank_date}       = cleandate($tr->address('.0.0'));
 	$trans{effective_date}  = cleandate($tr->address('.1.0'), $trans{bank_date});
-	$trans{type}            = $tr->address('.2.0.0');
-	$trans{thirdparty_name} = $tr->address('.2.0.2');
-	my $comments_str        = $tr->address('.2.0.4');
-	$trans{comments_str}    = $comments_str; # debugging
+	$trans{type}            = ltrim($tr->address('.2.0.0'));
+	$trans{thirdparty_name} = ltrim($tr->address('.2.0.2'));
 	$trans{amount}          = cleanamount($tr->address('.3.0'));
 	$trans{balance}         = cleanamount($tr->address('.4.0'));
 	$trans{infourl}         = $tr->address('.2.0')->attr('href');
-print Dumper(\%trans);	
-	my $badcomments=0;
-	# Split comments_str out into a lines
-	while (length $comments_str) {
-	    warn "Oddly-wrapped comments: <$comments_str>"
-	      and $badcomments++
-	      unless substr($comments_str, 0, 1, '') eq ' ';
-	    my $this =substr($comments_str, 0, 27, '');
-	    $this .= " " x (27-length($this));
-	    push @{$trans{comments}}, $this;
-	}
 	
-	# Because of bugs on the server, this doesn't always fetch data for
-	# the correct transaction.  (And wrong is worse then incomplete.)
-	if ($badcomments) {
-	    $agent->get($trans{infourl});
-	    $agent->follow(1) or      # frame name=
-	      warn $self->parse_error($agent);
-	    my $tree=HTML::TreeBuilder->new;
-	    $tree->no_space_compacting(1); # because norisbank is annoying.
-	    print "AC: " . $agent->content, "\n";
-	    $tree->parse($agent->content);
-	    $tree->eof;
-	    $tree->elementify;
-	    print $tree->dump;
+	# 2... doesn't work in perl5, sniff.
+	for (2..999) {
+	    my $elem = $tr->address('.2.0.'.($_*2));
+	    last if !$elem;
 	    
-	    my @data = $tree->look_down(_tag=>'td', class=>qr/label|lightgrey|bgsoll/);
-#	    print "Bad data: " .Dumper(\@data);
-	    
-	    my @labels;
-	    my $thislabel;
-	    foreach my $d (@data) {
-		print "Bad data: ";
-		$d->dump;
-		push @labels, $d->as_text and next if $d->attr('class') eq 'label';
-		$thislabel = shift @labels if @labels;
-		
-		$d = $d->as_text;
-		s/\xA0//;
-		
-		push @{$trans{extradata}{$thislabel}}, $d;
-	    }
-	    
-	    $trans{firstparty_name}=$trans{extradata}{"Auftraggeber/Zahlungspflichtiger"}[0];
-	    $trans{firstparty_blz} =$trans{extradata}{Bankleitzahl}[0];
-	    $trans{thirdparty_blz} =$trans{extradata}{Bankleitzahl}[1];
-	    $trans{firstparty_accntnum}=$trans{extradata}{Kontonummer}[0];
-	    $trans{thirdparty_accntnum}=$trans{extradata}{Kontonummer}[1];
-	    $trans{comments}=$trans{extradata}{Verwendungszweck};
-	    # Other data from the details page is also on the summary, or (currency) is unreliable and useless.
-	    $tree->delete;
-	    $self->find_page('umsatz_auskunft_giro')
-	      or die "Couldn't find page";
+	    push @{$trans{comments}}, ltrim($elem);
 	}
 	
+dprint Dumper(\%trans);
+
 	# Translate transaction types
 	local $_=$trans{type};
 	s/^.berweisung$/liability transfer (local init)/i;      # Customer-initiated transfer
@@ -349,22 +307,70 @@ print Dumper(\%trans);
 	s/^Scheck/check/i;                            # check
 	$trans{type}=$_;
 	
-	$trans{misc2} and ($trans{misc2} =~ s/^ // 
-	    or warn "No leading space on misc2?");
-	
-	return unless defined ($trans{comments}[0]);
-	grok_cardreader(\%trans);
-	grok_longnumber(\%trans);
-	grok_visa(\%trans);
+	if (defined ($trans{comments}[0])) {
+	    grok_cardreader(\%trans);
+	    grok_longnumber(\%trans);
+	    grok_visa(\%trans);
+	    grok_ga(\%trans);
+	}
 
 	\%trans;	
     } @trs;
 
     $self->status("Parsed transactions.");
-    return @transactions;
-#    return reverse @transactions;
+#    return @transactions;
+    return reverse @transactions;
 }
 
+=item blz
+
+    $account->blz
+    Finance::Bank::Norisbank::->blz
+    $account->BLZ
+    Finance::Bank::Norisbank::->BLZ
+
+Returns the Bankleihtzahl (sort code) of the bank.
+
+=cut
+
+sub blz {
+    return '76026000';
+}
+
+*BLZ = *blz;
+
+=item iban
+
+   $account->iban
+   $account->IBAN
+
+Returns the IBAN (International Bank Account Number) for this account.
+
+=cut
+
+sub iban {
+    return 'DE6276026000'.$_[0]->accountnr;
+}
+
+*IBAN = *iban;
+
+=item swift
+
+   $account->swift
+   Finance::Bank::Norisbank::->swift
+
+Returns the SWIFT-BIC (Society for Worldwide Interbank Financial Telecommunications - Bank Identification Code) of 
+Norisbank.  This is aliased to SWIFT as well.  (This returns the part of the SWIFT code that is the same for all Norisbank 
+branches.)
+
+=cut
+
+sub swift {
+    return 'NORS DE 71';
+}
+
+*SWIFT=*swift;
+
 # Aus Sicherheitgr.nden wurden Sie vom System abgemeldet.
 
 # Given a page with a possible error message on it, return the error
@@ -399,147 +405,12 @@ sub parse_error {
 
     $error =~ s/Die eingegebenen Datumsangaben sind nicht plausibel\./The date range is not possible/;
     $error =~ s/Aus Sicherheitgr.nden wurden Sie vom System abgemeldet\./Login Timeout/;
+    $error =~ s/Die Anzahl der Ums.tze, die pro Aufruf angezeigt werden k.nnen, ist auf 100 St.k.*/Too many records returned, truncating at 100./;
     $self->{error} = $error;  
     
     return $error;
 }
 
-# Given the content of the frameset page, figure out what the name of the page 
-# is.
-sub get_page_name
-{
-    my $self = shift;
-    my $content = shift;
-    
-    my $tree = HTML::TreeBuilder->new();
-    $tree->no_space_compacting(1); # because norisbank is annoying.
-    $tree->parse($self->{agent}->content);
-    $tree->eof;
-    $tree->elementify;
-    
-    my $page = $tree->look_down(_tag => 'frame', name => 'topFrame');
-    
-    return undef if(!$page);
-    
-    $page = $page->attr('src');
-    $page =~ s/.*screen=(\w+).*/$1/;
-    
-    print "get_page_name: $page\n";
-    
-    return lc($page||'');
-}
-
-
-# Finds the frameset, then goes back (via the link at the side) until we find
-# the page asked for.
-sub find_page
-{
-    my $self = shift;
-    my $name = shift;
-    
-    # Make sure we start with a frameset, or get_page_name will punt.
-    $self->find_frameset;
-    return if($self->get_page_name eq $name);
-    
-    print "Finding page: $name\n";
-    
-    $self->find_frameset;
-    $self->{agent}->follow_link(name=>'rightFrame') or
-      warn $self->parse_error($self->{agent});
-    
-    my $tree = HTML::TreeBuilder->new();
-    $tree->no_space_compacting(1); # because norisbank is annoying.
-    $tree->parse($self->{agent}->content);
-    $tree->eof;
-    $tree->elementify;
-    
-#    print "find_page, top: ", $tree->dump, "\n";
-    print "find_page, top: ", $tree->dump, "\n";
-    
-    my @hrefs = $tree->look_down(_tag => 'a');
-    my $page = '';
-    
-    print "find_page: found", 0+@hrefs, "\n";
-    
-    foreach my $href (@hrefs)
-    {
-	my $uri = URI->new($href->attr('href'));
-	print "find_page: uri=$uri\n";
-	my $screen = $uri->query_param('screen');
-	
-	next unless $screen =~ /^$name/i;
-	
-	# FIXME: Doesn't get($href->attr('href')) DTRT?
-	$self->{agent}->follow_link(url => $href->attr('href'));
-	
-	if($self->parse_error($self->{agent}->content))
-	{
-	    if($self->{error} =~ /Login Timeout/)
-	    {
-		# FIXME: This won't work.  We're at the beginning, we can't
-		# go backward to get forward.
-		$self->login();
-		return $self->find_page($name, $self->{agent}->content);
-	    }
-	    else
-	    {
-		return undef;
-	    }
-	}
-	
-	return 1;
-    }
-    
-    # If we havn't exited yet, then no a tags were found.  This is a very
-    # bad thing -- perhaps we were somehow on a framset?
-    return undef;
-}
-
-# Finds the main frameset by calling ->{agent}->back until we get there.
-sub find_frameset
-{
-    my $self = shift;
-    my $content = $self->{agent}->content;
-    
-    my $tree = HTML::TreeBuilder->new();
-    $tree->no_space_compacting(1); # because norisbank is annoying.
-    $tree->parse($content);
-    $tree->eof;
-    $tree->elementify;
-    
-    my $frameset;
-
-#    until($frameset = $tree->look_down('_tag', 'frameset') &&
-    until($frameset = $tree->look_down('_tag' => 'frame', 'name' => 'topFrame'))
-    {
-	$tree->dump;
-	$self->{agent}->back();
-	$tree->delete;
-	$tree = HTML::TreeBuilder->new();
-	$tree->no_space_compacting(1); # because norisbank is annoying.
-	$tree->parse($self->{agent}->content);
-	$tree->eof;
-	$tree->elementify;
-    
-	if($self->parse_error($self->{agent}->content))
-	{
-	    if($self->{error} =~ /Login Timeout/)
-	    {
-		$self->login();
-	    }
-	    else
-	    {	      
-		return undef;
-	    }
-	}
-    }
-    print "Found frameset: $frameset <" . $frameset->dump . ">\n";
-    $tree->delete;
-    
-    $self->{page} = $self->get_page_name($self->{agent}->content());
-    print "Page: ", $self->{page}, "\n";
-}
-    
 
 sub grok_longnumber {
   my $trans=shift;
@@ -547,7 +418,7 @@ sub grok_longnumber {
   return unless $trans->{comments}[0] =~ /^\d{27}$/;
   
   # 012345678901234567890123456
-  # DDMMHHMM...........SSSSLLLL
+  # DDMMHHMMTTTTTTTyymmSSSSLLLL
   
   local $_=$trans->{comments}[0];
   my $day   = substr($_, 0, 2);
@@ -555,7 +426,7 @@ sub grok_longnumber {
   my $hour  = substr($_, 4, 2);
   my $min   = substr($_, 6, 2);
   # FIXME: Format does not provide for year (?), so assume this year.
-  # FIXME: Assume same year as bank_date
+  # FIXME: Assume same year as bank_date?  Assume occourance that happens before bank_date?
   my $year  = (localtime)[5];
   local $ENV{TZ}="Europe/Rome"; # middle-european time.
   
@@ -565,8 +436,16 @@ sub grok_longnumber {
     $trans->{sale_date}=timelocal(0,$min,$hour,$day,$month-1,$year);
   }
   
-  # Bunch of ??? digits here
-  $trans->{__longnumber_unknown} = substr($_, 8, 11);
+  # minimal calls this the transaction number, media markt calls it the POS number
+  $trans->{transaction_number}=substr($_, 8, 7);
+    
+  my ($expireyear, $expiremonth) = (substr($_, 15, 2), substr($_, 17, 2));
+  dprint "Expires $expiremonth 20$expireyear?\n";
+  
+  # Netto gives us zeros for most fields, which kills us for this case.
+  if ($expireyear > 0 && $expiremonth > 0) {
+      $trans->{card_expiry} = timelocal(0,0,0,1,$expiremonth-1, $expireyear+100);
+  }
   
   $trans->{store_number}   =substr($_, 19, 4)+0;
   $trans->{checkout_number}=substr($_, 23, 4)+0;
@@ -576,12 +455,12 @@ sub grok_cardreader {
   my $trans=shift;
   
   foreach (@{$trans->{comments}}) {
-    if ($_ =~ m/^(EC |POZ|OLV)(\d{8}) (\d\d)\.(\d\d) (\d\d).(\d\d) (...)$/) {
+    if ($_ =~ m/^(EC |POZ|OLV|ELV)(\d{8}) (\d\d)\.(\d\d) (\d\d).(\d\d) (...)$/) {
       my %types = (
 		   'EC ' => 'Electronic Cash',
 		   'POZ' => 'POZ',
 		   'OLV' => 'OLV',
-		   
+	           'ELV' => 'ELV'
 		  );
       $trans->{ccard_type} = $types{$1};
       $trans->{cardreader} = $1.$2;
@@ -597,6 +476,7 @@ sub grok_cardreader {
   }
 }
 
+my $currency_info;
 sub grok_visa {
     my $trans=shift;
     
@@ -616,23 +496,68 @@ sub grok_visa {
     
     $trans->{misc}[1]=$misc2;
     $trans->{country}=$country;
-    my $currency = Locale::Object::Country
-      ->new(code_alpha2=>$country)
-      ->currency
-      ->code;
+    if (!$currency_info) {
+	# Load currency info once, keep it in memory thereafter.  It's only
+	# about 33k, so this is best-of-both-worlds, and doing it ourselves
+	# can keep us from depending on DBI, SQLite, and DateTime (though
+	# we may want to use DateTime anyway in the future).	
+	
+        # The database should be in the same directory as this file. Get
+        # the location.
+	my (undef, $path) = File::Spec->splitpath(__FILE__);
+	my $dbpath = $path . 'currency.storable';
+	$currency_info = Storable::retrieve($dbpath);
+	if (!$currency_info) {
+	    die "Couldn't retrieve $dbpath: $!";
+	}
+    }
+    my $currency = $currency_info->{lc $country};
     $trans->{currency} = $currency;
     $trans->{base_amount} = cleannumber($amt);
     
-    if (my ($rate) = $trans->{comments}[2] =~
-	m/^KURS ZU EURO *?(\d*,\d*)$/) {
+    for (@{$trans->{comments}}) {
+	if (my ($rate) = m/^KURS ZU EURO *?(\d*,\d*)$/) {
 	    $trans->{exchange_rate}=cleannumber($rate);
 	}
+    }
     
-    if (my ($fees) = $trans->{comments}[3] =~
-	# Note: Does not end in a $.
-	m/^GEBUEHR (\d*,\d\d)/) {
-	    $trans->{fees}=cleannumber($fees);
-	}
+    for (@{$trans->{comments}}) {
+	if (my ($fees) = 
+	    # Note: Does not end in a $.
+	    # I suspect the extra space before GEBUEHR that sometimes appears
+	    # is a bug either in this module or at the bank.
+	    m/^ ?GEBUEHR *(\d*[.,]\d\d)/) {
+		$trans->{fees}=cleannumber($fees);
+	    }
+    }
+}
+
+sub grok_ga {
+  my $trans=shift;
+  
+  return unless (my ($num, $blz, $unk) = 
+		 $trans->{thirdparty_name} =~
+		 /^GA NR(\d{8}) BLZ(\d{8}) (\d)$/);
+  $trans->{atm_number}=$num;
+  $trans->{atm_blz}=$blz;
+  #???
+  $trans->{card_number}=$unk;
+
+  $trans->{type}='ATM';
+
+  return unless (my ($day, $month, $hour, $minute, $name) =
+		 $trans->{comments}[0] =~
+		 m|^(\d\d)\.(\d\d)/(\d\d)\.(\d\d)UHR (.*)|);
+  local $ENV{TZ} = "Europe/Rome";
+  $trans->{store_date}=timelocal(0,$minute,$hour,$day,$month-1,(localtime)[5]);
+  
+  $trans->{atm_name}=$name;
+
+  return unless (my ($amt, $fee) =
+		 $trans->{comments}[1] =~
+		 /^EUR([ \d,.]*)[ *]GEB\.EUR([ \d]\d[,.]\d\d)$/);
+  $trans->{base_amount}=cleannumber($amt);
+  $trans->{fees}=cleannumber($fee);
 }
 
     
@@ -659,18 +584,27 @@ sub cleannumber {
 
 sub cleandate {
     local $_=shift;
-#    print "Date: $_\n";
+    croak "cleandate(undef)" if not defined $_;
+#    dprint "Date: $_\n";
     tr|.-/|---|;
     tr|-0-9||cd;
     my ($day,$mon,$year)=split(/-/);
-    if (not length $year) {
+    if (not $year) {
 	$year = (localtime)[5];
     }
     $_=timelocal(0,0,0,$day,$mon-1,$year);
-#    print "Date: $_\n";
+#    dprint "Date: $_\n";
     return $_;
 }
 
+# Remove leading/left whitespace
+sub ltrim {
+    local $_=shift;
+    return undef if not defined $_;
+    s/^\s*//;
+    $_;
+}
+
 1;
 
 =head1 AUTHOR
@@ -683,9 +617,13 @@ Known bugs: we aren't always able to get the verwindungszweck from the bank.
 This is because Norisbank's online banking is highly obnixious.  Working code 
 to fix this is very much welcome.
 
-The grok_* routines are based on guesswork.  There are still many unknowns, purticularly in the "longnumber" style.  If you think you have them figured out, or you think there is an error in my interpretation of them, please let me know.  If you find a standard for them, please, please, please let me know.
+The grok_* routines are based on guesswork.  If you think you have them
+figured out, or you think there is an error in my interpretation of them,
+please let me know.  If you find a standard for them, please, please,
+please let me know.
 
-The best place to tell me about bugs or feature requests is probably on rt.cpan.org.
+The best place to tell me about bugs or feature requests is probably on
+rt.cpan.org.
   
 =cut
 
